#!/usr/bin/env python

# Copyright (C) 2012, 2014 Moritz Huetten and Torsten Albrecht

# create MISMIP+ config override file

import argparse

parser = argparse.ArgumentParser(description='Create MISMIP+ config override file.',
                                 formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument('-A', '--softness', metavar='soft', type=float, default=6.338e-25,
                    help='Ice softness')

args = parser.parse_args()

A=args.softness
print A

try:
    from netCDF4 import Dataset as NC
except:
    print "netCDF4 is not installed!"
    sys.exit(1)

filename = "MISMIP+_conf.nc"

print '  creating MISMIP+_conf.nc for configuration parameters (overrides) ...'

nc = NC(filename, 'w', format="NETCDF3_CLASSIC")

var = nc.createVariable("pism_overrides", 'i')

attrs = {"geometry.update.use_basal_melt_rate": "yes",
         "flow_law.isothermal_Glen.ice_softness": A,
         "constants.ice.density": 918.,
         "constants.sea_water.density": 1028.,
         "constants.standard_gravity": 9.81,
         "bootstrapping.defaults.geothermal_flux": 0.0,
         "ocean.sub_shelf_heat_flux_into_ice": 0.0,
         "stress_balance.sia.bed_smoother.range": 0.0,
         "stress_balance.sia.max_diffusivity": 5000.0
         }

for name, value in attrs.iteritems():
    var.setncattr(name, value)

nc.close()
