#!/usr/bin/env python

# Copyright (C) 2015 Johannes Feldmann

from scipy.interpolate import BPoly
import sys
import getopt
import math
#from numpy import *
import numpy as np
# import scipy
# from scipy.interpolate import PiecewisePolynomial
try:
    from netCDF4 import Dataset as NC
except:
    print "netCDF4 is not installed!"
    sys.exit(1)
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pylab as plt


# geometry setup MISMIP3D Stnd-experiment


WRIT_FILE = 'MISMIP+_InitialSetup.nc'

boxWidth = 1.0 # km
accumrate = 0.3  # accumulation rate in m/a
wc = 24.0 # channel width in km
dc = 500 # channel depth in m
dy = 150 # reverse slope height in m
yo = 0 # basal topography offset in m
vs = 1 # vertical scaling of thickness and topography
hs = 1 # horizontal (longitudinal) scaling of topography
ss = 1 # horizontal scaling of bed slope
xpp = 700.0 # x coordinate of pinning point (km)
ypp = 160.0 # y coordinate of pinning point (km)
wpp = 15.0 # width of pinning point (km)
hpp = 20.0 # surface elevation of pinning point (m)
tpp = 1.0 # True/False 1.0/0.0 decides whether pinning point exists or not

#### command line arguments ####
try:
    opts, args = getopt.getopt(sys.argv[1:], "a:r:w:d:y:yo:vs:hs:ss:xpp:ypp:wpp:hpp:", ["accumrate=", "resolution=", "channelwidth=", "channeldepth=", "slopeheight=", "yo=", "vs=", "hs=", "ss=", "xpp=", "ypp=", "wpp=", "hpp=", "tpp="])
    for opt, arg in opts:
        if opt in ("-a", "--accumulation"):
            accumrate = arg
        if opt in ("-r", "--resolution"):  # resolution in km
            boxWidth = arg
        if opt in ("-w", "--channelwidth"):  # channel width in km
            wc = arg
        if opt in ("-d", "--channeldepth"):  # channel depth in m
            dc = arg
        if opt in ("-y", "--slopeheight"):  # elevation difference of reverse slope in m
            dy = arg
        if opt in ("--yo"):  # basal topography offset in m
            yo = arg
        if opt in ("--vs"):  # vertical scaling
            vs = arg
        if opt in ("--hs"):  # horizontal scaling
            hs = arg
        if opt in ("--ss"):  # bed slope scaling
            ss = arg
        if opt in ("--xpp"):  # x coordinate of pinning point
            xpp = arg
        if opt in ("--ypp"):  # y coordinate of pinning point
            ypp = arg
        if opt in ("--wpp"):  # width of pinning point
            wpp = arg
        if opt in ("--hpp"):  # surface elevation of pinning point
            hpp = arg
        if opt in ("--tpp"):  # true/false existence of pinning point
            tpp = arg
except getopt.GetoptError:
    print 'Incorrect command line arguments'
    sys.exit(2)


######## geometry setup (johannes.feldmann@pik) #########

boxWidth = float(boxWidth)
accumrate = float(accumrate)
wc = float(wc)
dc = float(dc)
dy = float(dy)
yo = float(yo)
vs = float(vs)
hs = float(hs)
ss = float(ss)
xpp = float(xpp)
ypp = float(ypp)
wpp = float(wpp)
hpp = float(hpp)
tpp = float(tpp)


### CONSTANTS ###

rho_i = 918. #kg/m3 ice density
rho_w = 1028.

Lx = hs*800 #km Domain length (along ice flow)
Ly = 320 #km Domain width (across ice flow)

xExtent = 2 * Lx # setup symmetric around x=0
yExtent = Ly 

xfront = hs*780.0  # x-position of fixed calving front in km
yfront = Ly / 2 

xfront2 = hs*200.0
# grid size: # of boxes

ny = int(np.floor(yExtent / boxWidth / 2) * 2 + 1)  # make it an odd number
nx = int(np.floor(xExtent / boxWidth / 2) * 2 + 1)  # make it an odd number

# grid size: extent in km's, origin (0,0) in the center of the domain

x = np.linspace(-xExtent / 2, xExtent / 2, nx) * 1000.0
y = np.linspace(0, yExtent, ny) * 1000.0

nxcenter = int(np.floor(0.5 * nx))
nycenter = int(np.floor(0.5 * ny))
nxfront = int(xfront / boxWidth)
nxfront2 = int(xfront2 / boxWidth)
nyfront = int(yfront / boxWidth)

thk = np.zeros((ny, nx))
topg = np.zeros((ny, nx))
topg2 = np.zeros((ny, nx))
Bx   = np.zeros((ny,nx))
Tx   = np.zeros((ny,nx))
By   = np.zeros((ny,nx))
ice_surface_temp = np.zeros((ny, nx))
precip = np.zeros((ny, nx))
ocean_kill_mask = np.zeros((ny, nx))

# print range(0,int(np.floor(0.5*nx)))

print "Informations from createSetup_Stnd.py:"
print "grid size (pixel):"
print ny
print nx
print "grid size center:"
print nxcenter
print nycenter

print "domain range in meters:"
print "x-dir:"
print x[0]
print x[nx - 1]
print "y-dir:"
print y[0]
print y[ny - 1]

# define bedrock geometry topg:

# x-direction
# B0 = -150.0 #m Bathymetry at x=0
# B2 = -728.8 #m Second bathymetry coefficient
# B4 = 343.91 #m Third bathymetry coefficient
# B6 = -50.57 #m Forth bathymetry coefficient
# xb = 300.0 #km Characteristic along-flow length scale of bathymetry
# xt = x/(xb*1000.0)

# for i in range(0,ny):
#     Bx[i,:] = B0 + B2*xt**2 + B4*xt**4 + B6*xt**6 # in m!

# T0 = -150.0 #m Bathymetry at x=0
# T2 = -728.8 #m Second bathymetry coefficient
# T4 = 343.91 #m Third bathymetry coefficient
# T6 = -50.57 #m Forth bathymetry coefficient
# tb = 300.0 #km Characteristic along-flow length scale of bathymetry
# tt = x/(tb*1000.0)

# for i in range(0,ny):
#     Tx[i,:] = T0 + T2*tt**2 + T4*xt**4 + T6*tt**6 # in m!

x0 = 0.0
x1 = hs*2.0*10**5
x2 = hs*5.0*10**5
dx12 = x2-x1
x1 = x2 - ss*dx12
x3 = hs*7.0*10**5
x4 = hs*8.0*10**5

y0 = -400.0
# y0 = -150.0
y1 = -600 - dy
# y1 = -750.0
y2 = -600.0
y3 = -1000.0

y3prime = 100./150000.
# y3prime = 0.0

# xin = np.array([x0,x1,x11,x2,x3])
# yin = np.array([[y0,0],[y1,0],[y11,0.0003],[y2,0],[y3,-y3prime]])
xin = np.array([x0,x1,x2,x4])
yin = np.array([[y0,0],[y1,0],[y2,0],[y3,-y3prime]])
# yin = np.array([[y0,0],[y1,0],[y2,0],[y3,-y3prime]])

xRHS = x[nxcenter::]

# interpolation
# interpRHS = interpolate.PiecewisePolynomial(xin,yin,xRHS,orders=[3,3,2],der=0) + offset
BPoly = BPoly.from_derivatives(xin,yin)

# mirror interpolated function
interpRHS = BPoly(xRHS)
interpLHS = map(None,interpRHS[1::])
interpLHS.reverse()


for j in range(0,ny):
    Bx[j,0:nxcenter] = interpLHS
    Bx[j,nxcenter::] = interpRHS
    # topg2[:,0:nxcenter] = interpLHS
    # topg2[:,nxcenter::] = interpRHS

# horizontal scaling
Bx = vs*Bx

# y-direction
fc = 4.0 #km Charecteristic width of the side walls of the channel
# dc = 250 #m Depth of the trough compared with the side walls

for j in range(0,nx):
    By[:,j] = dc/(1+np.exp(-2.0*(y/1000.-Ly/2.0-wc)/fc)) + dc/(1+np.exp(2.0*(y/1000.-Ly/2.0+wc)/fc))

Bmax = 800 #m

# Bx[:,np.where(x>x3)[0]] = -Bmax
# Bx[:,np.where(x<-x3)[0]] = -Bmax
# Bmax = 720 + np.zeros_like(Bx) #m
# topg = max(Bx + By, -Bmax)

Bxy = Bx + By
Txy = Tx + By

Txy[Txy<-Bmax] = -Bmax
# Bxy[Bxy<-Bmax] = -Bmax

yo = 500. - dc
# yo = -100

# topg = vs*Bxy
Bxy += yo

### add pinning point
XRHS, Y = np.meshgrid(xRHS,y)
xpp_ind = np.where(x==xpp*1000)[0][0]
ypp_ind = np.where(y==ypp*1000)[0][0]
Hpp = -Bxy[ypp_ind,xpp_ind] + hpp # vertical length of pinning point center (relative to local bed topography)

BppRHS = tpp * Hpp * np.exp(((-(XRHS/1000.-xpp)**2 - (Y/1000.-ypp)**2) / (2 * wpp**2)))

Bpp = np.concatenate((BppRHS[:,1:][:,::-1],BppRHS),axis=1)

topg = Bxy + Bpp


# topg2 = Txy

# define constant initial ice-thickness and extent:
initthk_yo0 = 2000.0
# initthk_vs0 = 2000.0
# initthk_yo0 = 3000.0

# initthk = vs*initthk_vs0 
initthk = initthk_yo0 - yo*rho_w/rho_i
# initthk = initthk_yo0 + (dc-500)*rho_w/rho_i
# initthk = 2000.0 - topg
# thk[:,nxcenter-nxfront:nxcenter+nxfront+1] = 0.1
# thk[:,nxcenter-nxfront2:nxcenter+nxfront2+1] = initthk[:,nxcenter-nxfront2:nxcenter+nxfront2+1]
# thk[:,nxcenter-nxfront:nxcenter+nxfront+1] = initthk[:,nxcenter-nxfront:nxcenter+nxfront+1]
thk[:,nxcenter-nxfront:nxcenter+nxfront+1] = initthk
ocean_kill_mask[:,nxcenter-nxfront:nxcenter+nxfront+1] = 1

# define precipitation field:
# precip[:,nxcenter-nxfront:nxcenter+nxfront+1] = accumrate * rho_i
precip[thk!=0] = accumrate * rho_i
# defining dummy temperature:
ice_surface_temp = 268.15 # K


##### define dimensions in NetCDF file #####
ncfile = NC(WRIT_FILE, 'w', format='NETCDF3_CLASSIC')
xdim = ncfile.createDimension('x', nx)
ydim = ncfile.createDimension('y', ny)

##### define variables, set attributes, write data #####
# format: ['units', 'long_name', 'standard_name', '_FillValue', array]

vars = {'y':   	['m',
                 'y-coordinate in Cartesian system',
                 'projection_y_coordinate',
                 None,
                 y],
        'x':   	['m',
                 'x-coordinate in Cartesian system',
                 'projection_x_coordinate',
                 None,
                 x],
        'thk': 	['m',
                 'ice thickness',
                 'land_ice_thickness',
                 -1.0,
                 thk],
        'topg': ['m',
                 'bedrock surface elevation',
                 'bedrock_altitude',
                 6000.0,
                 topg],
        'ice_surface_temp': ['K',
                             'annual mean air temperature at ice surface',
                             'surface_temperature',
                             -1.0,
                             ice_surface_temp],
        'climatic_mass_balance': ['kg m-2 year-1',
                                  'mean annual net ice equivalent accumulation rate',
                                  'land_ice_surface_specific_mass_balance_flux',
                                  -10.0,
                                  precip],
        'land_ice_area_fraction_retreat': ['',
                                           'ocean kill mask',
                                           'land_ice_area_fraction_retreat',
                                           -1.0,
                                           ocean_kill_mask],
        }

for name in vars.keys():
    [_, _, _, fill_value, data] = vars[name]
    if name in ['x', 'y']:
        var = ncfile.createVariable(name, 'f4', (name,))
    else:
        var = ncfile.createVariable(name, 'f4', ('y', 'x'), fill_value=fill_value)
    for each in zip(['units', 'long_name', 'standard_name'], vars[name]):
        if each[1]:
            setattr(var, each[0], each[1])
    var[:] = data

# finish up
ncfile.close()
print "NetCDF file ", WRIT_FILE, " created"
print ''

# plt.close()

# plt.figure(1)
# plt.plot(x,topg[nycenter,:])
# plt.ylim(-1000,-100)
# # plt.plot(x,topg2[nycenter,:])

# plt.grid()

# outname = 'topgX_dy%s_ss%s.pdf' %(dy,ss)
# plt.savefig(outname, bbox_inches='tight')
# print 'Saving figure "%s"' %outname

# plt.figure(2)
# plt.plot(y,topg[:,nxcenter])
# plt.plot(y,topg[:,nxcenter+200])
# plt.plot(y,topg[:,nxcenter+400])
# plt.plot(y,topg[:,nxcenter+600])
# plt.ylim(-1000,1000)
# # plt.plot(x,topg2[nycenter,:])

# plt.grid()

# outname = 'topgY_dc%s.pdf' %dc
# plt.savefig(outname, bbox_inches='tight')
# print 'Saving figure "%s"' %outname

